@@ -1,5 +1,5 @@
 /*
- * $Id: hid-ups.c,v 1.8 2003/09/21 19:14:10 kerns Exp $
+ * $Id: hid-ups.c,v 1.10 2004/08/26 09:20:06 kerns Exp $
  *
  *  Copyright (c) 2001 Vojtech Pavlik <vojtech@ucw.cz>
  *  Copyright (c) 2001 Paul Stewart <hiddev@wetlogic.net>
@@ -33,6 +33,8 @@
 #define DEBUG 1 		      /* if set prints full reports */
 #define TESTING 1		      /* if set disables actual operation */
 
+#define HID_MAX_USAGES 1024
+
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
@@ -170,6 +172,7 @@ struct s_ups_info {
     { 0xFF860005, T_NONE,   "APCGeneralCollection" },
     { 0xFF860013, T_NONE,   "APC860013_SetMinReturn?" },
     { 0xFF860016, T_APCDATE,"APCBattReplacementDate" },
+    { 0xFF860019, T_UNITS,  "APCBattCapBeforeRestart" },
     { 0xFF860023, T_NONE,   "APC860023_??????" },
     { 0xFF860024, T_NONE,   "APC860024_??????" },
     { 0xFF860025, T_NONE,   "APC860025_??????" },
