@@ -305,17 +305,23 @@
 #define    APC_CMD_ForceShutdown          0
 #define CI_DelayBeforeShutdown           62
 #define    APC_CMD_DelayBeforeShutdown    0
+#define CI_APCBattCapBeforeStartup       63
+#define    APC_CMD_BattCapBeforeStartup   0
+#define CI_APCDelayBeforeStartup         64
+#define    APC_CMD_DelayBeforeStartup     0
+#define CI_APCDelayBeforeShutdown        65
+#define    APC_CMD_DelayBeforeShutdown    0
 
 /* Items below this line are not "probed" for */
-#define CI_CYCLE_EPROM         63     /* Cycle programmable EPROM values */
+#define CI_CYCLE_EPROM         66     /* Cycle programmable EPROM values */
 #define    APC_CMD_CYCLE_EPROM    '-'
-#define CI_UPS_CAPS            64     /* Get UPS capabilities (command) string */
+#define CI_UPS_CAPS            67     /* Get UPS capabilities (command) string */
 #define    APC_CMD_UPS_CAPS       'a'
 /* ^^^^^^^^^^ see below if you change this ^^^^^^ */
 /* set to last command index. CHANGE!!! when adding new code.  */
 /* vvvvvvvvvv change here vvvvvvvvvvvvvvvvvvvvvvv */
 #define CI_MAXCI         CI_UPS_CAPS      /* maximum UPS commands we handle */
-#define CI_MAX_CAPS      CI_DelayBeforeShutdown
+#define CI_MAX_CAPS      CI_APCDelayBeforeShutdown
 
 #define GO_ON_BATT              'W'
 #define GO_ON_LINE              'X'
@@ -526,7 +532,7 @@
 #define Dmsg7(lvl, msg, a1, a2, a3, a4, a5, a6, a7) d_msg(__FILE__, __LINE__, lvl, msg, a1, a2, a3, a4, a5, a6, a7)
 #define Dmsg8(lvl, msg, a1, a2, a3, a4, a5, a6, a7, a8) d_msg(__FILE__, __LINE__, lvl, msg, a1, a2, a3, a4, a5, a6, a7, a8)
 #define Dmsg11(lvl,msg,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11) d_msg(__FILE__,__LINE__,lvl,msg,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11)
-void d_msg(char *file, int line, int level, char *fmt,...);
+void d_msg(const char *file, int line, int level, const char *fmt,...);
 #else
 #define Dmsg0(lvl, msg)
 #define Dmsg1(lvl, msg, a1)
@@ -555,6 +561,9 @@ void d_msg(char *file, int line, int level, char *fmt,...);
            strerror(errstat)); \
    } while(0)
 
+#else
+#define P(x)
+#define V(x)
 #endif
 
 /*
