@@ -5,28 +5,23 @@
  */
 
 /*
- *			 GNU GENERAL PUBLIC LICENSE
- *			    Version 2, June 1991
- *
- *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
- *			       675 Mass Ave, Cambridge, MA 02139, USA
- *  Everyone is permitted to copy and distribute verbatim copies
- *  of this license document, but changing it is not allowed.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+   Copyright (C) 2000-2004 Kern Sibbald
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of
+   the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with this program; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
+
  */
 
 #include "apc.h"
@@ -77,7 +72,17 @@ static void print_valid_eeprom_values(UPSINFO *ups);
 
 static void do_usb_testing(void);
 #ifdef HAVE_USB_DRIVER
+/* From linux-usb.c */
+bool write_int_to_ups(UPSINFO *ups, int ci, int value, char *name);
+bool read_int_from_ups(UPSINFO *ups, int ci, int* value);
+int usb_ups_get_capabilities(UPSINFO *ups);
+/* Our own test functions */
 static void usb_kill_power_test(void);
+static void usb_get_self_test_result(void);
+static void usb_run_self_test(void);
+static int usb_get_battery_date(void);
+static void usb_set_battery_date(void);
+static void usb_get_manf_date(void);
 #endif
 
 static void strip_trailing_junk(char *cmd);
@@ -108,6 +113,7 @@ static void pmsg(char *fmt,...)
     avsnprintf(buf, sizeof(buf), (char *) fmt, arg_ptr);
     va_end(arg_ptr);
     printf("%s", buf);
+    fflush(stdout);
     write_file(buf);
 }
 
@@ -188,7 +194,7 @@ void apctest_error_cleanup(UPSINFO *ups)
  *  and exits. It is normally called from the Error_abort
  *  define, which inserts the file and line number.
  */
-void apctest_error_out(char *file, int line, char *fmt,...)
+void apctest_error_out(const char *file, int line, const char *fmt,...)
 {
     char      buf[256];
     va_list   arg_ptr;
@@ -207,7 +213,7 @@ void apctest_error_out(char *file, int line, char *fmt,...)
 /* subroutine error_exit simply prints the supplied error
  * message, cleans up, and exits
  */
-void apctest_error_exit(char *fmt,...)
+void apctest_error_exit(const char *fmt,...)
 {
     char      buf[256];
     va_list   arg_ptr;
@@ -1524,30 +1530,57 @@ static void do_usb_testing(void)
    int quit = FALSE;
 
    pmsg("Hello, this is the apcupsd Cable Test program.\n\
-This part of apctest is for testing USB UPSes.\n\
-Please select the function you want to perform.\n");
+This part of apctest is for testing USB UPSes.\n");
+
+   pmsg("\nGetting UPS capabilities...");
+   if (!usb_ups_get_capabilities(ups))
+      pmsg("FAILED\nSome or all tests may not work!\n");
+   else
+      pmsg("SUCCESS\n");
+
+   pmsg("\nPlease select the function you want to perform.\n");
 
    while (!quit) {
       pmsg( "\n\
-1) Test 1 - kill UPS power\n\
-2) Quit\n\n");
+1) Test kill UPS power\n\
+2) Perform self-test\n\
+3) Read last self-test result\n\
+4) Change battery date\n\
+5) View battery date\n\
+6) View manufacturing date\n\
+7) Quit\n\n");
 
       cmd = get_cmd("Select function number: ");
       if (cmd) {
-	 int item = atoi(cmd);
-	 switch (item) {
-	 case 1:
-	    usb_kill_power_test();
-	    break;
-	 case 2:
-	    quit = TRUE;
-	    break;
-	 default:
-            pmsg("Illegal response. Please enter 1-2\n");
-	    break;
-	 }
+         int item = atoi(cmd);
+         switch (item) {
+         case 1:
+            usb_kill_power_test();
+            break;
+          case 2:
+            usb_run_self_test();
+            break;
+          case 3:
+            usb_get_self_test_result();
+            break;
+          case 4:
+            usb_set_battery_date();
+            break;
+          case 5:
+            usb_get_battery_date();
+            break;
+          case 6:
+            usb_get_manf_date();
+            break;
+          case 7:
+            quit = TRUE;
+            break;
+          default:
+            pmsg("Illegal response. Please enter 1-7\n");
+            break;
+         }
       } else {
-         pmsg("Illegal response. Please enter 1-2\n");
+         pmsg("Illegal response. Please enter 1-7\n");
       }
    }
    ptime();
@@ -1577,6 +1610,228 @@ Please enter any character when ready to continue: ");
    pmsg("returned from kill_power function.\n");
 }
 
+static void usb_get_self_test_result(void)
+{
+   int result;
+   if (!read_int_from_ups(ups, CI_ST_STAT, &result))
+   {
+      pmsg("\nI don't know how to run a self test on your UPS\n\
+or your UPS does not support self test.\n");
+      return;
+   }
+   
+   pmsg("Result of last self test: ");
+   switch(result)
+   {
+      case 1:
+         pmsg("PASSED\n");
+         break;
+      case 2:
+         pmsg("WARNING\n");
+         break;
+      case 3:
+         pmsg("ERROR\n");
+         break;
+      case 4:
+         pmsg("ABORTED\n");
+         break;
+      case 5:
+         pmsg("IN PROGRESS\n");
+         break;
+      case 6:
+         pmsg("NO TEST PERFORMED\n");
+      break;
+      default:
+         pmsg("UNKNOWN (%02x)\n", result);
+         break;
+   }
+}
+
+static void usb_run_self_test(void)
+{
+   int result;
+   int timeout;
+   
+   pmsg("\nThis test instructs the UPS to perform a self-test\n\
+operation and reports the result when the test completes.\n");
+
+   if (!read_int_from_ups(ups, CI_ST_STAT, &result))
+   {
+      pmsg("\nI don't know how to run a self test on your UPS\n\
+or your UPS does not support self test.\n");
+      return;
+   }
+   
+   pmsg("\nClearing previous self test result...");
+   if (!write_int_to_ups(ups, CI_ST_STAT, 0, "SelftestStatus"))
+   {
+      pmsg("FAILED\n");
+      return;
+   }
+   
+   for (timeout=0; timeout<10; timeout++)
+   {
+      if (!read_int_from_ups(ups, CI_ST_STAT, &result))
+      {
+         pmsg("FAILED\n");
+         return;
+	   }
+	
+      if (result == 6)
+      {
+         pmsg( "CLEARED\n");
+         break;
+      }
+
+      sleep(1);
+   }
+   
+   if (timeout == 10)
+   {
+      pmsg("FAILED\n");
+      return;
+   }
+
+   pmsg("Initiating self test...");
+   if (!write_int_to_ups(ups, CI_ST_STAT, 1, "SelftestStatus"))
+   {
+      pmsg("FAILED\n");
+      return;
+   }
+
+   pmsg("INITIATED\n");
+   
+   pmsg("Waiting for test to complete...");
+   
+   for (timeout=0; timeout<20; timeout++)
+   {
+      if (!read_int_from_ups(ups, CI_ST_STAT, &result))
+      {
+         pmsg("ERROR READING STATUS\n");
+         write_int_to_ups(ups, CI_ST_STAT, 3, "SelftestStatus");
+         return;
+      }
+   
+      if (result != 6)
+      {
+         pmsg( "COMPLETED\n");
+         break;
+      }
+      sleep(1);
+   }
+   
+   if (timeout == 20)
+   {
+      pmsg("TEST DID NOT COMPLETE\n");
+      write_int_to_ups(ups, CI_ST_STAT, 3, "SelftestStatus");
+      return;
+   }
+
+   usb_get_self_test_result();
+}
+
+static int usb_get_battery_date(void)
+{
+   int result;
+   if (!read_int_from_ups(ups, CI_BATTDAT, &result))
+   {
+      pmsg("\nI don't know how to access the battery date on your UPS\n\
+or your UPS does not support the battery date feature.\n");
+      return 0;
+   }
+   
+   /*
+    * Date format is:
+    *   YYYY|YYYM|MMMD|DDDD
+    *   bit  0-4:  Day
+    *        5-8:  Month
+    *        9-15: Year-1980
+    */
+   pmsg("Current battery date: %02u/%02u/%04u\n",
+      (result & 0x1e0) >> 5,
+      result & 0x1f,
+      1980 + ((result & 0xfe00) >> 9));
+
+   return result;
+}
+
+static void usb_set_battery_date(void)
+{
+   char* cmd;
+   int result, day, month, year, temp, max;
+   if (!(result = usb_get_battery_date()))
+      return;
+   
+   cmd = get_cmd("Enter new battery date (MM/DD/YYYY): ");
+   if (!isdigit(cmd[0]) || !isdigit(cmd[1]) || cmd[2] != '/' ||
+       !isdigit(cmd[3]) || !isdigit(cmd[4]) || cmd[5] != '/' ||
+       !isdigit(cmd[6]) || !isdigit(cmd[7]) || !isdigit(cmd[8]) ||
+       !isdigit(cmd[9]) || cmd[10] != '\0' ||
+       ((month = strtoul(cmd, NULL, 10)) > 12) || (month < 1) ||
+       ((day = strtoul(cmd+3, NULL, 10)) > 31) || (day < 1) ||
+       ((year = strtoul(cmd+6, NULL, 10)) < 1980) )
+   {
+       pmsg("Invalid format.\n");
+       return;
+   }
+
+   result = ((year-1980) << 9) | (month << 5) | day;
+
+   pmsg("Writing new date...");
+   if (!write_int_to_ups(ups, CI_BATTDAT, result, "ManufactureDate"))
+   {
+      pmsg("FAILED\n");
+      return;
+   }
+      
+   pmsg("SUCCESS\n");
+
+   pmsg("Waiting for change to take effect...");
+   for (max=0; max<10; max++)
+   {
+      if (!read_int_from_ups(ups, CI_BATTDAT, &temp))
+      {
+         pmsg("ERROR\n");
+         return;
+      }
+
+      if (temp == result)
+         break;
+
+      sleep(1);
+   }
+
+   if (max == 10)
+      pmsg("TIMEOUT\n");
+   else
+      pmsg("SUCCESS\n");
+
+   usb_get_battery_date();
+}
+
+static void usb_get_manf_date(void)
+{
+   int result;
+   if (!read_int_from_ups(ups, CI_MANDAT, &result))
+   {
+      pmsg("\nI don't know how to access the manufacturing date on your UPS\n\
+or your UPS does not support the manufacturing date feature.\n");
+      return;
+   }
+
+   /*
+    * Date format is:
+    *   YYYY|YYYM|MMMD|DDDD
+    *   bit  0-4:  Day
+    *        5-8:  Month
+    *        9-15: Year-1980
+    */
+   pmsg("Manufacturing date: %02u/%02u/%04u\n",
+      (result & 0x1e0) >> 5,
+      result & 0x1f,
+      1980 + ((result & 0xfe00) >> 9));
+}
+
 #endif
 
 /*
