@@ -6,17 +6,42 @@
  *
  *   Parts of this code (especially the initialization and walking
  *     the reports) were derived from a test program hid-ups.c by:    
- *
- *     Copyright (c) 2001 Vojtech Pavlik <vojtech@ucw.cz>
- *     Copyright (c) 2001 Paul Stewart <hiddev@wetlogic.net>
- *
+ *	 Vojtech Pavlik <vojtech@ucw.cz>
+ *	 Paul Stewart <hiddev@wetlogic.net>
  */
 
+/*
+   Copyright (C) 2001-2004 Kern Sibbald
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of
+   the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with this program; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
+
+ */
+
+
+/*
+ * The following is a work around for a problem in 2.6 kernel
+ *  linux/hiddev.h file that should be fixed in 2.6.8
+ */
+#define HID_MAX_USAGES 1024
 
 #include "apc.h"
 #include "usb.h"
 #include <math.h>
 
+
 #include <asm/types.h>
 #include <linux/hiddev.h>
 
@@ -61,6 +86,7 @@ static bool linkcheck = false;
 #define P_OUTPUT  0x84001c	     /* output values */
 #define P_BATTERY 0x840012	     /* battery values */
 #define P_INPUT   0x84001a	     /* input values */
+#define P_PWSUM   0x840024           /* power summary */
 
 /* No Command Index, don't save this value */
 #define CI_NONE -1
@@ -74,7 +100,7 @@ static bool linkcheck = false;
  *   Entries in the table with ci == CI_NONE are not used, for the
  *   moment, they are retained just so they are not forgotten.
  */
-struct s_known_info {
+static const struct s_known_info {
     int ci;			      /* command index */
     unsigned usage_code;	      /* usage code */
     unsigned physical;		      /* physical usage */
@@ -115,6 +141,7 @@ struct s_known_info {
     { CI_Manufacturer,	      0x8400fd, P_ANY,	  T_INDEX   },	/* iManufacturer */
     { CI_UPSMODEL,	      0x8400fe, P_ANY,	  T_INDEX   },	/* iProduct */
     { CI_SERNO, 	      0x8400ff, P_ANY,	  T_INDEX   },	/* iSerialNumber */
+    { CI_MANDAT,	      0x850085, P_PWSUM,  T_DATE    },	/* ManufactureDate */
 
     /*	Page 0x85 is the Battery System Page */
     { CI_RemCapLimit,	      0x850029, P_ANY,	  T_CAPACITY},	/* RemCapLimit */
@@ -131,7 +158,7 @@ struct s_known_info {
     { CI_CycleCount,	      0x85006b, P_ANY,	  T_NONE    },
     { CI_BattPackLevel,       0x850080, P_ANY,	  T_NONE    },	/* BattPackLevel */
     { CI_NONE,		      0x850083, P_ANY,	  T_CAPACITY},	/* DesignCapacity */
-    { CI_MANDAT,	      0x850085, P_ANY,	  T_DATE    },	/* ManufactureDate */
+    { CI_BATTDAT,	      0x850085, P_BATTERY,T_DATE    },	/* ManufactureDate */
     { CI_IDEN,		      0x850088, P_ANY,	  T_INDEX   },	/* iDeviceName */
     { CI_NONE,		      0x850089, P_ANY,	  T_INDEX   },	/* iDeviceChemistry */
     { CI_NONE,		      0x85008b, P_ANY,	  T_NONE    },	/* Rechargeable */
@@ -144,14 +171,17 @@ struct s_known_info {
     { CI_NONE,		      0x8500db, P_ANY,	  T_NONE    },	/* VoltageNotRegulated */
 
     /*	Pages 0xFF00 to 0xFFFF are vendor specific */
-    { CI_STATUS,	      0xFF860060, P_ANY,  T_BITS    },	/* APCStatusFlag */
-    { CI_DSHUTD,	      0xFF860076, P_ANY,  T_UNITS   },	/* APCShutdownAfterDelay */
-    { CI_NONE,		      0xFF860005, P_ANY,  T_NONE    },	/* APCGeneralCollection */
-    { CI_APCForceShutdown,    0xFF86007C, P_ANY,  T_NONE    },	/* APCForceShutdown */
-    { CI_NONE,		      0xFF860072, P_ANY,  T_NONE    },	/* APCPanelTest */
-    { CI_BattReplaceDate,     0xFF860016, P_ANY,  T_APCDATE },	/* APCBattReplaceDate */
-    { CI_NONE,		      0xFF860042, P_ANY,  T_NONE    },	/* APC_UPS_FirmwareRevision */
-    { CI_NONE,		      0xFF860079, P_ANY,  T_NONE    },	/* APC_USB_FirmwareRevision */
+    { CI_STATUS,                  0xFF860060, P_ANY,  T_BITS    },  /* APCStatusFlag */
+    { CI_DSHUTD,                  0xFF860076, P_ANY,  T_UNITS   },  /* APCShutdownAfterDelay */
+    { CI_NONE,                    0xFF860005, P_ANY,  T_NONE    },  /* APCGeneralCollection */
+    { CI_APCForceShutdown,        0xFF86007C, P_ANY,  T_NONE    },  /* APCForceShutdown */
+    { CI_NONE,                    0xFF860072, P_ANY,  T_NONE    },  /* APCPanelTest */
+    { CI_BattReplaceDate,         0xFF860016, P_ANY,  T_APCDATE },  /* APCBattReplaceDate */
+    { CI_NONE,                    0xFF860042, P_ANY,  T_NONE    },  /* APC_UPS_FirmwareRevision */
+    { CI_NONE,                    0xFF860079, P_ANY,  T_NONE    },  /* APC_USB_FirmwareRevision */
+    { CI_APCBattCapBeforeStartup, 0xFF860019, P_ANY,  T_CAPACITY},  /* APCBattCapBeforeStartup */
+    { CI_APCDelayBeforeStartup,   0xFF86007E, P_ANY,  T_UNITS   },  /* APCDelayBeforeStartup */
+    { CI_APCDelayBeforeShutdown,  0xFF86007D, P_ANY,  T_UNITS   },  /* APCDelayBeforeShutdown */
 };
 #define KNOWN_INFO_SZ (sizeof(known_info)/sizeof(known_info[0]))
 
@@ -294,7 +324,7 @@ static int open_usb_device(UPSINFO *ups)
     }
 
     if (my_data->orig_device[0] == 0) {
-       strcpy(my_data->orig_device, ups->device);
+       astrncpy(my_data->orig_device, ups->device, sizeof(my_data->orig_device));
     }
     astrncpy(name, my_data->orig_device, sizeof(name));
     p = strchr(name, '[');
@@ -345,7 +375,7 @@ static int open_usb_device(UPSINFO *ups)
 	   break;
        }
        if (my_data->fd >= 0) {
-	  strcpy(ups->device, devname);
+	  astrncpy(ups->device, devname, sizeof(ups->device));
 	  return 1;
        }
        sleep(1);
@@ -384,7 +414,7 @@ auto_detect:
 
 auto_opened:
     if (my_data->fd >= 0) {
-       strcpy(ups->device, devname);
+       astrncpy(ups->device, devname, sizeof(ups->device));
        return 1;
     } else {
        ups->device[0] = 0;
@@ -467,7 +497,7 @@ static int find_usb_application(UPSINFO *ups)
 /*
  * Get a field value
  */
-static int get_value(UPSINFO *ups, int ci, USB_INFO *uinfo)
+static bool get_value(UPSINFO *ups, int ci, USB_INFO *uinfo)
 {
     struct hiddev_string_descriptor sdesc;
     struct hiddev_report_info rinfo;
@@ -476,16 +506,16 @@ static int get_value(UPSINFO *ups, int ci, USB_INFO *uinfo)
     int exponent;
 
     if (!ups->UPS_Cap[ci] || !my_data->info[ci]) {
-	return 0;		      /* UPS does not have capability */
+	return false;		      /* UPS does not have capability */
     }
     info = my_data->info[ci];	      /* point to our info structure */
     rinfo.report_type = info->uref.report_type;
     rinfo.report_id = info->uref.report_id;
     if (ioctl(my_data->fd, HIDIOCGREPORT, &rinfo) < 0) {  /* update Report */
-       return 0;
+       return false;
     }
     if (ioctl(my_data->fd, HIDIOCGUSAGE, &info->uref) < 0) {  /* update UPS value */
-       return 0;
+       return false;
     }
     exponent = info->unit_exponent;
     if (exponent > 7) {
@@ -493,11 +523,11 @@ static int get_value(UPSINFO *ups, int ci, USB_INFO *uinfo)
     }
     if (info->data_type == T_INDEX) { /* get string */
 	if (info->uref.value == 0) {
-	   return 0;
+	   return false;
 	}
 	sdesc.index = info->uref.value;
 	if (ioctl(my_data->fd, HIDIOCGSTRING, &sdesc) < 0) {
-	    return 0;
+	    return false;
 	}
 	strncpy(ups->buf, sdesc.value, ups->buf_len);
 	info->value_type = V_STRING;
@@ -558,7 +588,7 @@ static int get_value(UPSINFO *ups, int ci, USB_INFO *uinfo)
 	      exponent, info->dValue, ci);
     }
     memcpy(uinfo, info, sizeof(USB_INFO));
-    return 1;
+    return true;
 }
 
 /*
@@ -742,13 +772,19 @@ int usb_ups_open(UPSINFO *ups)
        reinitialize_private_structure(ups);
     }
     if (my_data->orig_device[0] == 0) {
-       strcpy(my_data->orig_device, ups->device);
+       astrncpy(my_data->orig_device, ups->device, sizeof(my_data->orig_device));
     }
     if (!open_usb_device(ups)) {
 	write_unlock(ups);
-        Error_abort1(_("Cannot open UPS device: \"%s\" --\n"
-           "For a link to detailed USB troubleshooting information,\n"
-           "please see <http://www.apcupsd.com/support.html>.\n"), ups->device);
+	if (ups->device[0]) {
+           Error_abort1(_("Cannot open UPS device: \"%s\" --\n"
+              "For a link to detailed USB trouble shooting information,\n"
+              "please see <http://www.apcupsd.com/support.html>.\n"), ups->device);
+	} else {
+           Error_abort0(_("Cannot find UPS device --\n"
+              "For a link to detailed USB trouble shooting information,\n"
+              "please see <http://www.apcupsd.com/support.html>.\n"));
+	}
     }
     clear_ups(UPS_SLAVE);
     write_unlock(ups);
@@ -788,12 +824,11 @@ int usb_ups_get_capabilities(UPSINFO *ups)
     struct hiddev_usage_ref uref;
     int i, j, k, n;
 
-    write_lock(ups);
     if (ioctl(my_data->fd, HIDIOCINITREPORT, 0) < 0) {
-       write_unlock(ups);
        Error_abort1("Cannot init USB HID report. ERR=%s\n", strerror(errno));
     }
 
+    write_lock(ups);
     /*
      * Walk through all available reports and determine
      * what information we can use.
@@ -803,7 +838,6 @@ int usb_ups_get_capabilities(UPSINFO *ups)
 	rinfo.report_id = HID_REPORT_ID_FIRST;
 	while (ioctl(my_data->fd, HIDIOCGREPORTINFO, &rinfo) >= 0) {
 	    for (i = 0; i < (int)rinfo.num_fields; i++) { 
-
 		memset(&finfo, 0, sizeof(finfo));
 		finfo.report_type = rinfo.report_type;
 		finfo.report_id = rinfo.report_id;
@@ -980,7 +1014,7 @@ int usb_ups_read_static_data(UPSINFO *ups)
     /* UPS_SERIAL_NUMBER */
     if (get_value(ups, CI_SERNO, &uinfo)) {
 	char *p;
-	strncpy(ups->serial, ups->buf, sizeof(ups->serial));
+	astrncpy(ups->serial, ups->buf, sizeof(ups->serial));
 	/*
 	 * If serial number has garbage, trash it.
 	 */
@@ -1131,20 +1165,20 @@ int usb_ups_read_volatile_data(UPSINFO *ups)
     if (get_value(ups, CI_ST_STAT, &uinfo)) {
 	switch (uinfo.uref.value) {
 	case 1: 		      /* passed */
-           strcpy(ups->X, "OK");
+           astrncpy(ups->X, "OK", sizeof(ups->X));
 	   break;
 	case 2: 		      /* Warning */
-           strcpy(ups->X, "WN");
+           astrncpy(ups->X, "WN", sizeof(ups->X));
 	   break;
 	case 3: 		      /* Error */
 	case 4: 		      /* Aborted */
-           strcpy(ups->X, "NG");
+           astrncpy(ups->X, "NG", sizeof(ups->X));
 	   break;
 	case 5: 		      /* In progress */
-           strcpy(ups->X, "IP");
+           astrncpy(ups->X, "IP", sizeof(ups->X));
 	   break;
 	case 6: 		      /* None */
-           strcpy(ups->X, "NO");
+           astrncpy(ups->X, "NO", sizeof(ups->X));
 	   break;
 	default:
 	   break;
@@ -1155,103 +1189,260 @@ int usb_ups_read_volatile_data(UPSINFO *ups)
     return 1;
 }
 
-static int write_int_to_ups(UPSINFO *ups, int ci, int value, char *name)
+bool read_int_from_ups(UPSINFO *ups, int ci, int* value)
+{
+    USB_INFO info;
+    if (!get_value(ups, ci, &info))
+    	return false;
+    	
+    *value = info.iValue;
+    return true;
+}
+
+bool write_int_to_ups(UPSINFO *ups, int ci, int value, char *name)
 {
     struct hiddev_report_info rinfo;
-    struct hiddev_field_info finfo;
     USB_DATA *my_data = (USB_DATA *)ups->driver_internal_data;
     USB_INFO *info;
     int old_value, new_value;
-    int i;
 
-    errno = 0; 
     if (ups->UPS_Cap[ci] && my_data->info[ci]) {
-	info = my_data->info[ci];	  /* point to our info structure */
-	rinfo.report_type = info->uref.report_type;
-	rinfo.report_id = info->uref.report_id;
-	if (ioctl(my_data->fd, HIDIOCGREPORTINFO, &rinfo) < 0) {  /* get Report */
-            Dmsg2(000, "HIDIOCGREPORTINFO for shutdown function %s failed. ERR=%s\n", 
-		  name, strerror(errno));
-	    return 0;
-	}
-        Dmsg1(100, "REPORTINFO num_fields=%d\n", rinfo.num_fields);
-	finfo.report_type = info->uref.report_type;
-	finfo.report_id = info->uref.report_id;
-	finfo.field_index = info->uref.field_index;
-        Dmsg7(100, "before FIELDINFO call: type=%d id=%d index=%d logical_min=%d \n\
-logical_max=%d exponent=%d unit=0x%x\n",
-	    finfo.report_type, finfo.report_id, finfo.field_index,
-	    finfo.logical_minimum, finfo.logical_maximum, 
-	    finfo.unit_exponent, finfo.unit);
-	if (ioctl(my_data->fd, HIDIOCGFIELDINFO, &finfo) < 0) {  /* Get field info */
-            Dmsg2(000, "HIDIOCGFIELDINFO for shutdown function %s failed. ERR=%s\n", 
-		  name, strerror(errno));
-	    return 0;
-	}
-        Dmsg7(100, "FIELDINFO type=%d id=%d index=%d logical_min=%d \n\
-logical_max=%d exponent=%d unit=0x%x\n",
-	    finfo.report_type, finfo.report_id, finfo.field_index,
-	    finfo.logical_minimum, finfo.logical_maximum, 
-	    finfo.unit_exponent, finfo.unit);
-        Dmsg3(100, "GUSAGE type=%d id=%d index=%d\n", info->uref.report_type,
-	   info->uref.report_id, info->uref.field_index);
-	if (ioctl(my_data->fd, HIDIOCGUSAGE, &info->uref) < 0) {  /* get UPS value */
-            Dmsg2(000, "HIDIOGSUSAGE for shutdown function %s failed. ERR=%s\n", 
-		  name, strerror(errno));
-	    return 0;
-	}
-	old_value = info->uref.value;
-	info->uref.value = value;
+        info = my_data->info[ci];         /* point to our info structure */
+        rinfo.report_type = info->uref.report_type;
+        rinfo.report_id = info->uref.report_id;
+
+        if (ioctl(my_data->fd, HIDIOCGREPORT, &rinfo) < 0) {  /* get Report */
+            Dmsg2(000, "HIDIOCGREPORT for function %s failed. ERR=%s\n", 
+                  name, strerror(errno));
+            return false;
+        }
+
+        if (ioctl(my_data->fd, HIDIOCGUSAGE, &info->uref) < 0) {  /* get UPS value */
+            Dmsg2(000, "HIDIOCGUSAGE for function %s failed. ERR=%s\n", 
+                  name, strerror(errno));
+            return false;
+        }
+        old_value = info->uref.value;
+
+        info->uref.value = value;
         Dmsg3(100, "SUSAGE type=%d id=%d index=%d\n", info->uref.report_type,
-	   info->uref.report_id, info->uref.field_index);
-	if (ioctl(my_data->fd, HIDIOCSUSAGE, &info->uref) < 0) {  /* update UPS value */
-            Dmsg2(000, "HIDIOCSUSAGE for shutdown function %s failed. ERR=%s\n", 
-		  name, strerror(errno));
-	    return 0;
-	}
-	for (i=0; i < 1; i++) {
-	   if (ioctl(my_data->fd, HIDIOCSREPORT, &rinfo) < 0) {  /* update Report */
-               Dmsg2(000, "HIDIOCSREPORT for shutdown function %s failed. ERR=%s\n", 
-		     name, strerror(errno));
-	       return 0;
-	   }
-	}
-	if (ioctl(my_data->fd, HIDIOCGUSAGE, &info->uref) < 0) {  /* get UPS value */
-            Dmsg2(000, "HIDIOCSUSAGE for shutdown function %s failed. ERR=%s\n", 
-		  name, strerror(errno));
-	    return 0;
-	}
-	new_value = info->uref.value;
-        Dmsg3(000, "Shutdown function %s ci=%d value=%d OK.\n", name, ci, value);
+           info->uref.report_id, info->uref.field_index);
+        if (ioctl(my_data->fd, HIDIOCSUSAGE, &info->uref) < 0) {  /* update UPS value */
+            Dmsg2(000, "HIDIOCSUSAGE for function %s failed. ERR=%s\n", 
+                  name, strerror(errno));
+            return false;
+        }
+
+        if (ioctl(my_data->fd, HIDIOCSREPORT, &rinfo) < 0) {  /* update Report */
+            Dmsg2(000, "HIDIOCSREPORT for function %s failed. ERR=%s\n", 
+                  name, strerror(errno));
+            return false;
+        }
+
+        /*
+         * This readback of the report is NOT just for debugging. It has the effect
+         * of flushing the above SET_REPORT to the device, which is important since
+         * we need to make sure it happens before subsequent reports are sent.
+         */
+
+        if (ioctl(my_data->fd, HIDIOCGREPORT, &rinfo) < 0) {  /* get Report */
+            Dmsg2(000, "HIDIOCGREPORT for function %s failed. ERR=%s\n", 
+                  name, strerror(errno));
+            return false;
+        }
+
+        if (ioctl(my_data->fd, HIDIOCGUSAGE, &info->uref) < 0) {  /* get UPS value */
+            Dmsg2(000, "HIDIOCGUSAGE for function %s failed. ERR=%s\n", 
+                  name, strerror(errno));
+            return false;
+        }
+
+        new_value = info->uref.value;
+        Dmsg3(100, "function %s ci=%d value=%d OK.\n", name, ci, value);
         Dmsg4(100, "%s before=%d set=%d after=%d\n", name, old_value, value, new_value);
-	return 1;
+        return true;
     }
-    return 0;
+    Dmsg2(000, "function %s ci=%d not available in this UPS.\n", name, ci);
+    return false;
 }
 
-#define CI_APCShutdownAfterDelay CI_DSHUTD
+/* How long to wait before killing output power */
+#define SHUTDOWN_DELAY  60 
+
+/* How many seconds of good utility power before turning output back on */
+#define STARTUP_DELAY   10
+
+#define UPS_HAS_CAP(ci) \
+    (ups->UPS_Cap[ci] && ((USB_DATA *)ups->driver_internal_data)->info[ci])
 
 int usb_ups_kill_power(UPSINFO *ups)
 {
+    char *func;
+    int shutdown = 0;
 
-    if (!usb_ups_get_capabilities(ups)) {
-       return 0;
+    Dmsg0(200, "Enter usb_ups_kill_power\n");
+    
+    /*
+     * We try various different ways to shutdown the UPS (i.e. killpower).
+     * Some of these commands are not supported on all UPSes, but that
+     * should cause no harm.
+     */
+
+    /*
+     * First, set required battery capacity before startup to 0 so UPS will not
+     * wait for the battery to charge before turning back on. Not all UPSes have
+     * this capability, so this setting is allowed to fail. The value we program
+     * here should be made configurable some day.
+     */
+    func = "CI_APCBattCapBeforeStartup";
+    if (!write_int_to_ups(ups, CI_APCBattCapBeforeStartup, 0, func)) {
+       Dmsg1(100, "Unable to set %s (not an error)\n", func);
     }
 
-    /* We try various different ways to shutdown the UPS (i.e. killpower).
-     *	 Some of these commands are not supported on all UPSes, but that
-     *	 should cause no harm.
+    /*
+     * Second, set the length of time to wait after power returns before starting
+     * up. We set it to something pretty low, but it seems the UPS rounds this
+     * value up to the nearest multiple of 60 seconds. Not all UPSes have this
+     * capability, so this setting is allowed to fail.  The value we program
+     * here should be made configurable some day.
      */
-    write_int_to_ups(ups, CI_DelayBeforeShutdown, 20, "CI_DelayBeforeShutdown");
+    func = "CI_APCDelayBeforeStartup";
+    if (!write_int_to_ups(ups, CI_APCDelayBeforeStartup, 10, func)) {
+       Dmsg1(100, "Unable to set %s (not an error)\n", func);
+    }
 
-    write_int_to_ups(ups, CI_ShutdownRequested, 1, "CI_ShutdownRequested");
+    /*
+     * SmartUPS shutdown
+     *
+     * If both DWAKE and DelayBeforeShutdown are available, trigger a hibernate
+     * by writing DWAKE a few seconds longer than DelayBeforeShutdown. ORDER IS
+     * IMPORTANT. The write to DelayBeforeShutdown starts both timers ticking
+     * down and the UPS will hibernate when DelayBeforeShutdown hits zero.
+     */
+    if (UPS_HAS_CAP(CI_DWAKE) && UPS_HAS_CAP(CI_DelayBeforeShutdown)) {
+        Dmsg0(000, "UPS appears to support SmartUPS style shutdown.\n");   
+        func = "CI_DWAKE";
+        if (!write_int_to_ups(ups, CI_DWAKE, SHUTDOWN_DELAY+4, func)) {
+           Dmsg1(000, "Kill power function \"%s\" failed.\n", func);   
+        }
+        else {
+            func = "CI_DelayBeforeShutdown";
+            if (!write_int_to_ups(ups, CI_DelayBeforeShutdown, SHUTDOWN_DELAY, func)) {
+               Dmsg1(000, "Kill power function \"%s\" failed.\n", func);   
+               write_int_to_ups(ups, CI_DWAKE, -1, "CI_DWAKE"); /* reset prev timer */
+            }
+            else {
+                shutdown = 1;
+            }
+        }
+    }
 
-    write_int_to_ups(ups, CI_APCShutdownAfterDelay, 30, "CI_APCShutdownAfterDelay");
+    /*
+     * BackUPS shutdown
+     *
+     * Alternately, if APCDelayBeforeShutdown is available, setting it will
+     * start a countdown after which the UPS will hibernate.
+     */
+    if(!shutdown && UPS_HAS_CAP(CI_APCDelayBeforeShutdown)) {
+        Dmsg0(000, "UPS appears to support BackUPS style shutdown.\n");   
+        func = "CI_APCDelayBeforeShutdown";
+        if (!write_int_to_ups(ups, CI_APCDelayBeforeShutdown, SHUTDOWN_DELAY, func)) {
+           Dmsg1(000, "Kill power function \"%s\" failed.\n", func);   
+        }
+        else {
+            shutdown = 1;
+        }
+    }
 
-    write_int_to_ups(ups, CI_APCForceShutdown, 1, "CI_APCForceShutdown");
+    /*
+     * All UPSes tested so far are covered by one of the above cases. However,
+     * there are a couple other ways to shutdown.
+     */
+
+    /*
+     * Misc method A
+     *
+     * Writing CI_DelayBeforeReboot starts a countdown timer, after which the UPS
+     * will hibernate. If utility power is out, the UPS will stay hibernating until
+     * power is restored. SmartUPSes seem to support this method, but PowerChute
+     * uses the dual countdown method above, so we prefer that one. UPSes seem to
+     * round the value up to 90 seconds if it is any lower. Note that the behavior
+     * described here DOES NOT comply with the standard set out in the HID Usage
+     * Tables for Power Devices spec. 
+     */
+    if(!shutdown && UPS_HAS_CAP(CI_DelayBeforeReboot)) {
+        Dmsg0(000, "UPS appears to support DelayBeforeReboot style shutdown.\n");   
+        func = "CI_DelayBeforeReboot";
+        if (!write_int_to_ups(ups, CI_DelayBeforeReboot, SHUTDOWN_DELAY, func)) {
+           Dmsg1(000, "Kill power function \"%s\" failed.\n", func);   
+        }
+        else {
+            shutdown = 1;
+        }
+    }
+
+    /*
+     * Misc method B
+     *
+     * We can set CI_APCForceShutdown to true (it's a boolean flag). We have no
+     * control over how long the UPS waits before turning off. Experimentally
+     * it seems to be about 60 seconds. Some BackUPS models support this in
+     * addition to the preferred BackUPS method above. It's included here
+     * "just in case".
+     */
+    if(!shutdown && UPS_HAS_CAP(CI_APCForceShutdown)) {
+        Dmsg0(000, "UPS appears to support ForceShutdown style shutdown.\n");   
+        func = "CI_APCForceShutdown";
+        if (!write_int_to_ups(ups, CI_APCForceShutdown, 1, func)) {
+           Dmsg1(000, "Kill power function \"%s\" failed.\n", func);   
+        }
+        else {
+            shutdown = 1;
+        }
+    }
+
+    /*
+     * Misc method C
+     *
+     * This one seems to turn the UPS off completely after a given delay.
+     * The only way to power the UPS back on is to manually hit the power button.
+     */
+    if(!shutdown && UPS_HAS_CAP(CI_DelayBeforeShutdown)){
+        Dmsg0(000, "UPS appears to support DelayBeforeShutdown style shutdown.\n");   
+        func = "CI_DelayBeforeShutdown";
+        if (!write_int_to_ups(ups, CI_DelayBeforeShutdown, SHUTDOWN_DELAY, func)) {
+           Dmsg1(000, "Kill power function \"%s\" failed.\n", func);   
+        }
+        else {
+            shutdown = 1;
+        }
+    }
+
+    /*
+     * I give up.
+     */
+    if (!shutdown) {
+        Dmsg0(000, "I don't know how to turn off this UPS...sorry.\n"
+                   "Please report this, along with the output from\n"
+                   "running examples/hid-ups, to the apcupsd-users\n"
+                   "mailing list (apcupsd-users@lists.sourceforge.net).\n");
+    }
+
+    /*
+     * Note that there are a couple other CIs that look interesting for shutdown,
+     * but they're not what we want.
+     *
+     * APCShutdownAfterDelay: Tells the UPS how many seconds to wait after power
+     *     goes out before asserting ShutdownRequested (see next item).
+     *
+     * CI_ShutdownRequested: This is an indicator from the UPS to the server
+     *     that it would like the server to begin shutting down. In conjunction
+     *     with APCShutdownAfterDelay this can be used to offload the decision of
+     *     when to shut down the server to the UPS.
+     */
 
     Dmsg0(200, "Leave usb_ups_kill_power\n");
-    return 1;
+    return shutdown;
 }
 
 int usb_ups_program_eeprom(UPSINFO *ups, int command, char *data)
